@@ -1,12 +1,15 @@
 package Crypt::MagicSignatures::Envelope;
 use strict;
 use warnings;
+
+use v5.10.1;
+
 use Crypt::MagicSignatures::Key qw/b64url_encode b64url_decode/;
 use Carp qw/carp croak/;
 use Mojo::DOM;
 use Mojo::JSON;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 # MagicEnvelope namespace
 use constant ME_NS => 'http://salmon-protocol.org/ns/magic-env';
@@ -590,30 +593,45 @@ Crypt::MagicSignatures::Envelope - MagicEnvelopes for the Salmon Protocol
 
 =head1 SYNOPSIS
 
+  use Crypt::MagicSignatures::Key;
   use Crypt::MagicSignatures::Envelope;
 
-  my $me = Crypt::MagicSignatures::Envelope->new({
+  # Create a new MagicKey for signing messages
+  my $mkey = Crypt::MagicSignatures::Key->new(size => 1024);
+
+  # Fold a new envelope
+  my $me = Crypt::MagicSignatures::Envelope->new(
     data => 'Some arbitrary string.'
-  });
+  );
 
-  $me->sign('key-01' => 'RSA.vsd...');
+  # Sign magic envelope
+  $me->sign($mkey);
 
-  if ($me->verify('RSA.vsd...')) {
+  # Extract the public key
+  my $mkey_public = $mkey->to_string;
+
+  # Verify the signature of the envelope
+  if ($me->verify($mkey_public)) {
     print 'Signature is verified!';
   };
 
+
 =head1 DESCRIPTION
 
-L<Crypt::MagicSignatures::Envelope> implements MagicEnvelopes with MagicSignatures as described in the
-L<MagicSignatures Specification|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html> to sign messages of the L<Salmon Protocol|http://www.salmon-protocol.org/>.
-MagicSignatures is a I<"robust mechanism for digitally signing nearly arbitrary messages">.
+L<Crypt::MagicSignatures::Envelope> implements MagicEnvelopes
+with MagicSignatures as described in the
+L<MagicSignatures Specification|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html>
+to sign messages of the
+L<Salmon Protocol|http://www.salmon-protocol.org/>.
+MagicSignatures is a
+I<"robust mechanism for digitally signing nearly arbitrary messages">.
 
 B<This module is an early release! There may be significant changes in the future.>
 
 
 =head1 ATTRIBUTES
 
-=head2 C<alg>
+=head2 alg
 
   my $alg = $me->alg;
 
@@ -621,7 +639,7 @@ The algorithm used for signing the MagicEnvelope.
 Defaults to C<RSA-SHA256>, which is the only supported algorithm.
 
 
-=head2 C<data>
+=head2 data
 
   my $data = $me->data;
   $me->data('Hello world!');
@@ -629,17 +647,18 @@ Defaults to C<RSA-SHA256>, which is the only supported algorithm.
 The decoded data folded in the MagicEnvelope.
 
 
-=head2 C<data_type>
+=head2 data_type
 
   my $data_type = $me->data_type;
   $me->data_type('text/plain');
 
-The mime type of the data folded in the MagicEnvelope.
+The Mime type of the data folded in the MagicEnvelope.
 Defaults to C<text/plain>.
 
 
-=head2 C<dom>
+=head2 dom
 
+  Fold an xml message
   my $me = Crypt::MagicSignatures::Envelope->new( data => <<'XML' );
   <?xml version='1.0' encoding='UTF-8'?>
   <entry xmlns='http://www.w3.org/2005/Atom'>
@@ -647,18 +666,19 @@ Defaults to C<text/plain>.
   </entry>
   XML
 
+  # Define an xml mime type
   $me->data_type('application/atom+xml');
 
-  # alice@example.com
   print $me->dom->at('author > uri')->text;
+  # alice@example.com
 
 The L<Mojo::DOM> object of the decoded data,
-if the MagicEnvelope contains XML.
+in the case the MagicEnvelope contains XML.
 
 B<This attribute is experimental and may change without warning!>
 
 
-=head2 C<encoding>
+=head2 encoding
 
   my $encoding = $me->encoding;
 
@@ -666,7 +686,7 @@ The encoding of the MagicEnvelope.
 Defaults to C<base64url>, which is the only encoding supported.
 
 
-=head2 C<signature>
+=head2 signature
 
   my $sig = $me->signature;
   my $sig = $me->signature('key-01');
@@ -682,15 +702,15 @@ and possibly a C<key_id>.
 If no matching signature is found, a C<false> value is returned.
 
 
-=head2 C<signature_base>
+=head2 signature_base
 
   my $base = $me->signature_base;
 
-The signature base string of the MagicEnvelope as described in the
-L<MagicSignatures Specification|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#sbs>.
+The L<signature base string|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#sbs>
+of the MagicEnvelope.
 
 
-=head2 C<signed>
+=head2 signed
 
   # With key id
   if ($me->signed('key-01')) {
@@ -714,7 +734,7 @@ MagicEnvelope was signed with this specific key.
 =head1 METHODS
 
 
-=head2 C<new>
+=head2 new
 
   $me = Crypt::MagicSignatures::Envelope->new(<<'MEXML');
   <?xml version="1.0" encoding="UTF-8"?>
@@ -732,16 +752,16 @@ MagicEnvelope was signed with this specific key.
   MEXML
 
 The constructor accepts MagicEnvelope data in various formats.
-It accepts MagicEnvelopes in the XML format or an
-XML document including a MagicEnvelope C<provenance> element
-as described in the
-L<MagicSignatures Specification|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#anchor4>.
+It accepts MagicEnvelopes in the
+L<XML format|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#anchor4>
+or an XML document including a
+L<MagicEnvelope provenance|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#anchor7>
+element.
 
-Additionally it accepts MagicEnvelopes in the JSON notation
+Additionally it accepts MagicEnvelopes in the
+L<JSON notation|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#anchor5>
 or defined by the same attributes as the JSON notation
-(but with the data not encoded)
-as described in the
-L<MagicSignatures Specification|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#anchor5>.
+(but with the data not encoded).
 The latter is the common way to fold new envelopes.
 
   $me = Crypt::MagicSignatures::Envelope->new(<<'MEJSON');
@@ -771,9 +791,8 @@ The latter is the common way to fold new envelopes.
     ]
   );
 
-Finally the constructor accepts MagicEnvelopes in the compact
-MagicEnvelope notation as described in the
-L<MagicSignatures Specification|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#compact>.
+Finally the constructor accepts MagicEnvelopes in the
+L<compact notation|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#compact>.
 
   $me = Crypt::MagicSignatures::Envelope->new(<<'MECOMPACT');
     bXktMDE=.S1VqYVlIWFpuRGVTX3l4S09CcWdjRVFDYVlu
@@ -784,28 +803,30 @@ L<MagicSignatures Specification|http://salmon-protocol.googlecode.com/svn/trunk/
   MECOMPACT
 
 
-=head2 C<sign>
+=head2 sign
 
   $me->sign('key-01' => 'RSA.hgfrhvb ...')
      ->sign('RSA.hgfrhvb ...')
      ->sign('RSA.hgfrhvb ...', -data)
      ->sign('key-02' => 'RSA.hgfrhvb ...', -data);
 
-  my $mkey = Crypt::MagicSignatures::Key->new('RSA.hgfrhvb ...')
+  my $mkey = Crypt::MagicSignatures::Key->new('RSA.hgfrhvb ...');
   $me->sign($mkey);
 
 Adds a signature to the MagicEnvelope.
 
 For adding a signature, the private key with an optional prepended
 key id has to be given.
-The private key can be a L<Crypt::MagicSignatures::Key> object,
-a MagicKey string as described in the
-L<MagicSignatures Specification|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#rfc.section.8.1> or a hash reference
+The private key can be a
+L<Crypt::MagicSignatures::Key> object,
+a MagicKey in
+L<compact notation|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#rfc.section.8.1>
+or a hash reference
 containing the non-generation parameters accepted by the
 L<Crypt::MagicSignatures::Key> constructor.
 Optionally a flag C<-data> can be passed,
-that will sign the data payload instead of the signature base string as defined in the
-L<MagicSignatures Specification|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#rfc.section.3.2>
+that will sign the data payload instead of the
+L<signature base string|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#rfc.section.3.2>
 I<(this is implemented for compatibility with non-standard implementations)>.
 
 On success, the method returns the MagicEnvelope,
@@ -816,9 +837,9 @@ A MagicEnvelope can be signed multiple times.
 B<This method is experimental and may change without warning!>
 
 
-=head2 C<verify>
+=head2 verify
 
-  my $mkey = Crypt::MagicSignatures::Key->new( 'RSA.hgfrhvb ...' )
+  my $mkey = Crypt::MagicSignatures::Key->new( 'RSA.hgfrhvb ...' );
 
   $me->verify(
     'RSA.vsd...',
@@ -828,49 +849,54 @@ B<This method is experimental and may change without warning!>
 
 Verifies a signed envelope against a bunch of given public MagicKeys.
 Returns a C<true> value on success, otherwise C<false>.
-
 If one key succeeds, the envelope is verified.
 
 An element can be a L<Crypt::MagicSignatures::Key> object,
-a MagicKey string as described in the
-L<MagicSignatures Specification|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#rfc.section.8.1> or a hash reference
+a MagicKey in
+L<compact notation|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#rfc.section.8.1>
+or a hash reference
 containing the non-generation parameters accepted by the
 L<Crypt::MagicSignatures::Key> constructor.
 
 For referring to a certain key, an array reference
-can be passed, containing the key (defined as described above) with an optional prepended key id and an optional flag appended,
+can be passed, containing the key (defined as described above)
+with an optional prepended key id and an optional flag appended,
 referring to the data to be verified.
 Conforming with the specification the default value is C<-base>,
-referring to the base signature string of the MagicEnvelope.
-C<-data> will verify against the data only, C<-compatible> will first try to verify against the base signature string and then will verify against the data on failure
+referring to the
+L<signature base string|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#rfc.section.3.2>
+of the MagicEnvelope.
+C<-data> will verify against the data only, C<-compatible> will first try to
+verify against the base signature string and then will
+verify against the data on failure
 I<(this is implemented for compatibility with non-standard implementations)>.
 
 B<This method is experimental and may change without warning!>
 
 
-=head2 C<to_compact>
+=head2 to_compact
 
   my $compact_string = $me->to_compact;
 
-Returns the MagicEnvelope in compact notation as described in the
-L<MagicSignatures Specification|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#compact>.
+Returns the MagicEnvelope in
+L<compact notation|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#compact>.
 
 
-=head2 C<to_json>
+=head2 to_json
 
   my $json_string = $me->to_json;
 
-Returns the MagicEnvelope as a stringified json representation as described in the
-L<MagicSignatures Specification|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#anchor5>.
+Returns the MagicEnvelope as a stringified
+L<json representation|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#anchor5>.
 
 
-=head2 C<to_xml>
+=head2 to_xml
 
   my $xml_string = $me->to_xml;
   my $xml_provenance_string = $me->to_xml(1);
 
-Returns the MagicEnvelope as a stringified xml representation as described in the
-L<MagicSignatures Specification|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#anchor4>.
+Returns the MagicEnvelope as a stringified
+L<xml representation|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#anchor4>.
 If a C<true> value is passed, a provenance fragment will be returned instead
 of a valid xml document.
 
@@ -887,7 +913,8 @@ The signing and verifification is not guaranteed to be
 compatible with other implementations!
 Implementations like L<StatusNet|http://status.net/> (L<Identi.ca|http://identi.ca/>),
 L<MiniMe|https://code.google.com/p/minime-microblogger/>, and examples from the
-L<reference implementation|https://code.google.com/p/salmon-protocol/source/browse/> are tested.
+L<reference implementation|https://code.google.com/p/salmon-protocol/source/browse/>
+are tested.
 
 See the test suite for further information.
 
@@ -899,7 +926,8 @@ See the test suite for further information.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2011-2013, Nils Diewald.
+Copyright (C) 2011-2013, L<Nils Diewald|http://nils-diewald.de/>.
+
 
 This program is free software, you can redistribute it and/or modify it under
 the same terms as Perl.
